@@ -3,19 +3,21 @@
  * int.c
  *	  Functions for the built-in integer types (except int8).
  *
- * Portions Copyright (c) 1996-2002, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/int.c,v 1.52 2002/08/22 00:01:43 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/utils/adt/int.c,v 1.57 2003/08/04 02:40:05 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 /*
  * OLD COMMENTS
  *		I/O routines:
- *		 int2in, int2out, int2vectorin, int2vectorout, int4in, int4out
+ *		 int2in, int2out, int2recv, int2send
+ *		 int4in, int4out, int4recv, int4send
+ *		 int2vectorin, int2vectorout, int2vectorrecv, int2vectorsend
  *		Conversion routines:
  *		 itoi, int2_text, int4_text
  *		Boolean operators:
@@ -32,6 +34,7 @@
 #include <ctype.h>
 #include <limits.h>
 
+#include "libpq/pqformat.h"
 #include "utils/builtins.h"
 
 #ifndef SHRT_MAX
@@ -69,6 +72,31 @@ int2out(PG_FUNCTION_ARGS)
 	PG_RETURN_CSTRING(result);
 }
 
+/*
+ *		int2recv			- converts external binary format to int2
+ */
+Datum
+int2recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_INT16((int16) pq_getmsgint(buf, sizeof(int16)));
+}
+
+/*
+ *		int2send			- converts int2 to binary format
+ */
+Datum
+int2send(PG_FUNCTION_ARGS)
+{
+	int16		arg1 = PG_GETARG_INT16(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendint(&buf, arg1, sizeof(int16));
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
 /*
  *		int2vectorin			- converts "num num ..." to internal form
  *
@@ -93,7 +121,10 @@ int2vectorin(PG_FUNCTION_ARGS)
 	while (*intString && isspace((unsigned char) *intString))
 		intString++;
 	if (*intString)
-		elog(ERROR, "int2vector value has too many values");
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+				 errmsg("int2vector has too many elements")));
+
 	while (slot < INDEX_MAX_KEYS)
 		result[slot++] = 0;
 
@@ -131,6 +162,37 @@ int2vectorout(PG_FUNCTION_ARGS)
 	PG_RETURN_CSTRING(result);
 }
 
+/*
+ *		int2vectorrecv			- converts external binary format to int2vector
+ */
+Datum
+int2vectorrecv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+	int16	   *result = (int16 *) palloc(sizeof(int16[INDEX_MAX_KEYS]));
+	int			slot;
+
+	for (slot = 0; slot < INDEX_MAX_KEYS; slot++)
+		result[slot] = (int16) pq_getmsgint(buf, sizeof(int16));
+	PG_RETURN_POINTER(result);
+}
+
+/*
+ *		int2vectorsend			- converts int2vector to binary format
+ */
+Datum
+int2vectorsend(PG_FUNCTION_ARGS)
+{
+	int16	   *int2Array = (int16 *) PG_GETARG_POINTER(0);
+	StringInfoData buf;
+	int			slot;
+
+	pq_begintypsend(&buf);
+	for (slot = 0; slot < INDEX_MAX_KEYS; slot++)
+		pq_sendint(&buf, int2Array[slot], sizeof(int16));
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
 /*
  * We don't have a complete set of int2vector support routines,
  * but we need int2vectoreq for catcache indexing.
@@ -173,6 +235,31 @@ int4out(PG_FUNCTION_ARGS)
 	PG_RETURN_CSTRING(result);
 }
 
+/*
+ *		int4recv			- converts external binary format to int4
+ */
+Datum
+int4recv(PG_FUNCTION_ARGS)
+{
+	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
+
+	PG_RETURN_INT32((int32) pq_getmsgint(buf, sizeof(int32)));
+}
+
+/*
+ *		int4send			- converts int4 to binary format
+ */
+Datum
+int4send(PG_FUNCTION_ARGS)
+{
+	int32		arg1 = PG_GETARG_INT32(0);
+	StringInfoData buf;
+
+	pq_begintypsend(&buf);
+	pq_sendint(&buf, arg1, sizeof(int32));
+	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+}
+
 
 /*
  *		===================
@@ -193,10 +280,10 @@ i4toi2(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 
-	if (arg1 < SHRT_MIN)
-		elog(ERROR, "i4toi2: '%d' causes int2 underflow", arg1);
-	if (arg1 > SHRT_MAX)
-		elog(ERROR, "i4toi2: '%d' causes int2 overflow", arg1);
+	if (arg1 < SHRT_MIN || arg1 > SHRT_MAX)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
 
 	PG_RETURN_INT16((int16) arg1);
 }
@@ -551,6 +638,11 @@ int4div(PG_FUNCTION_ARGS)
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 / arg2);
 }
 
@@ -611,6 +703,11 @@ int2div(PG_FUNCTION_ARGS)
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT16(arg1 / arg2);
 }
 
@@ -647,6 +744,11 @@ int24div(PG_FUNCTION_ARGS)
 	int16		arg1 = PG_GETARG_INT16(0);
 	int32		arg2 = PG_GETARG_INT32(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 / arg2);
 }
 
@@ -683,6 +785,11 @@ int42div(PG_FUNCTION_ARGS)
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 / arg2);
 }
 
@@ -692,6 +799,11 @@ int4mod(PG_FUNCTION_ARGS)
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 % arg2);
 }
 
@@ -701,6 +813,11 @@ int2mod(PG_FUNCTION_ARGS)
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT16(arg1 % arg2);
 }
 
@@ -710,6 +827,11 @@ int24mod(PG_FUNCTION_ARGS)
 	int16		arg1 = PG_GETARG_INT16(0);
 	int32		arg2 = PG_GETARG_INT32(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 % arg2);
 }
 
@@ -719,6 +841,11 @@ int42mod(PG_FUNCTION_ARGS)
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
 
+	if (arg2 == 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_DIVISION_BY_ZERO),
+				 errmsg("division by zero")));
+
 	PG_RETURN_INT32(arg1 % arg2);
 }
 
